*DECK NLSPAR
      SUBROUTINE NLSPAR(FUNC,M,N,X,SAC,STPMIN,MAXFUN,W,IW,IRN,IP,A,IRNA,
     ,IPA,HMAX,IE)
C
************************************************************************
**                                                                    **
**    NOTE THAT NLSPAR IS WRITTEN IN MODULAR FORM USING THE ADDITIONAL**
**    ROUTINES:                                                       **
**       NPLIN                                                        **
**       SUMSQ                                                         *
**       SUBNL                                                         *
**    NPLIN THROUGH SUBNL ACCOMPANY NLSPAR ON AELIB SOURCE TAPE UNDER **
**    DECK NAME NLSPAR                                                **
**                                                                    **
************************************************************************
C
C  PURPOSE
C
C      THIS ROUTINE SOLVES A SYSTEM OF NON-LINEAR EQUATIONS
C
C        R(I)[X(1),X(2),...X(N)] = F(I)[X(1),X(2),...X(N)]
C                                 + SUM(A(I,J) * X(J)) _ J = 1,2,...,N;
C                       FOR I = 1,2,...,M
C
C      WHERE M \ N  IN THE SENSE THAT IT SEARCHES FOR A LOCAL MINIMUM
C      OF THE SUM OF THE SQUARES
C
C        S = SUM((R(I)[X(1),X(2),...,X(N)]) ** 2) _ I = 1,2,...,N .
C
C      IT IS DESIGNED SPECIALLY FOR THE CASE WHERE THE MATRICES
C      [PF(I) / PX(J)] (JACOBIAN MATRIX) AND [A(I,J)] (LINEAR PART OF
C      THE EQUATION) ARE SPARSE (I. E. CONTAIN A LARGE NUMBER OF ZEROS).
C      THE USER MUST SUPPLY [A(I,J)], AN INITIAL ESTIMATE OF
C      X(1),X(2),...,X(N) AND A SUBROUTINE WHICH CALCULATES F(I) AND
C      (OPTIONALLY) EACH PF(I) / PX(J).
C
C
C  ARGUMENT LIST
C
C    FUNC     IS THE NAME OF A SUBROUTINE WRITTEN BY THE USER. IT MUST
C             HAVE THE FORM
C
C                  SUBROUTINE FUNC(M,N,X,F,D)
C                  DIMENSION  X(N), F(M), D(1)
C
C             IT MUST CALCULATE F(I)[X(1),X(2),...,X(N)], I = 1,2,...,M
C             AND STORE THEM IN ARRAY F.  IF DERIVATIVES ARE BEING
C             CALCULATED ANAYTICALLY (HMAX = 0.0) THEN THE NON-ZERO
C             DERIVATIVES PF(I) / PX(J) SHOULD BE CALCULATED AND STORED
C             BY COLUMNS IN ARRAY D; OTHERWISE D IS A DUMMY ARGUMENT.
C
C    M        IS AN INTEGER VARIABLE SPECIFYING THE NUMBER OF EQUATIONS.
C
C    N        IS AN INTEGER VARIABLE SPECIFYING THE NUMBER OF
C             INDEPENDENT VARIABLES X(J).  N  @  M.
C
C    X        IS A REAL ARRAY OF LENGTH N WHICH IS SET TO AN APPROXIMATE
C             SOLUTION.  ON RETURN IT CONTAINS THE BEST SOLUTION FOUND
C             BY NLSPAR.
C
C    SAC      IS A REAL VARIABLE SET TO AN ACCEPTABLE VALUE FOR THE SUM
C             OF THE SQUARES S (SEE ABOVE EQUATION).  A RETURN WILL BE
C             MADE AS SOON AS S @ SAC.  SAC MAY BE SET TO ZERO, IN WHICH
C             CASE RETURN IS CONTROLLED BY STPMIN ALONE.
C
C    STPMIN   IS A REAL VARIABLE SET TO INDICATE THE ACCURACY TO WHICH
C             THE USER WISHES TO CALCULATE X(1),X(2),...,X(N).  A RETURN
C             IS MADE IF AN ITERATION MAKES CHANGES SIGMA(I) TO X(I)
C             SUCH THAT //SIGMA// < STPMIN (WHERE //SIGMA// ** 2 =
C             SUM(SIGMA(I) ** 2) _ I = 1,2,...,N .  THE USER MAY SET
C             STPMIN TO ZERO SINCE A RETURN IS ALSO MADE IF //SIGMA//
C             @ EPS * //X// WHERE EPS IS THE RELATIVE MACHINE ACCURACY.
C
C    MAXFUN   IS AN INTEGER VARIABLE SET TO THE MAXIMUM NUMBER OF CALLS
C             OF FUNC ALLOWED.  AN ERROR RETURN IS MADE IF A SOLUTION
C             HAS NOT BEEN FOUND WITHIN THIS NUMBER OF CALLS.
C
C    W        IS A REAL ARRAY USED FOR WORKSPACE. ITS DIMENSION MUST BE
C             SPECIFIED IN IW AND TO INSURE ENOUGH SPACE IS AVAILABLE
C            SHOULD BE LARGER THAN 4*M + 7*N + 3*M*N + 5. AFTER A
C             *SUCCESSFUL* ENTRY W(IW) IS SET TO THE NUMBER OF REAL
C            VARIABLES ACTUALLY USED.
C
C    IW       IS AN INTEGER VARIABLE WHICH IS SET TO INDICATE THE
C             DIMENSION OF THE ARRAY W.
C
C    IRN(NRIRN) AND IP(NRIP)
C            INTEGER ARRAYS, DIMENSIONS NRIRN AND NRIP RESPECTIVELY, TO
C            DESCRIBE THE SPARSITY STRUCTURE OF THE JACOBIAN (J). IF THE
C           IRN AND IP ARE ALSO SET INTERNALLY.
C            HOWEVER, IF THE JACOBIAN ELEMENTS ARE TO BE SPECIFIED
C            ANALYTICALLY BY THE USER, THE JACOBIAN STRUCTURE MUST FIRST
C            BE SPECIFIED IN IRN AND IP, WHICH DEFINE THE LOCATION OF
C            NON-ZERO TERMS. THE NON-ZERO DERIVATIVES ARE STORED BY
C            COLUMNS.  E.G.
C
C    PF(1)/PX(1),PF(3)/PX(1),PF(6)/PX(1),...,PF(2)/PX(2),PF(4)/PX(2),...
C
C            IP(J) POINTS TO THE POSITION OF THE BEGINNING OF THE J-TH
C            COLUMN IN THIS ORDERING , UNLESS THIS COLUMN IS NULL, IN
C            WHICH CASE IP(J) EQUALS IP(J+1) AND IRN(K) INDICATES THE
C            ROW NUMBER OF THE K-TH NON-ZERO. IP(N+1)-1 EQUALS THE
C            NUMBER OF NON-ZERO DERIVATIVES.
C            NRIRN\ THE NUMBER OF NON-ZERO DERIVATIVES AND NRIP \ N+1.
C
C             TO SET UP THESE ARRAYS THE USER MAY CALL JACOB ROUTINE;
C             FUNC IS SUITABLE FOR USE IN CONJUNCTION WITH JACOB.
C
C    A        IS A REAL ARRAY SET TO CONTAIN THE NON-ZERO COEFFICIENTS
C             A(I,J), STORED BY COLUMNS (E. G. A(1,1), A(3,1), A(5,1),
C             A(2,2), A(4,2), A(1,3), A(5,3), A(3,4), A(5,5)).
C
C     IRNA(NRIRNA) AND
C       IPA(NRIPA)   INTEGER ARRAYS DIMENSION NRIRNA AND NRIPA RESPEC-
C            TIVELY. THESE ARE SET TO INDEX [A(I,J)] IN THE SAME WAY AS
C            IRN AND IP INDEX [ PF(I)/PX(J) ]. THE SPECIAL CASE WHERE
C            A(I,J)=0 FOR ALL I AND J MAY BE INDICATED BY SETTING IPA(1)
C            =0. (FOR THE MATRIX A ABOVE :
C            IRNA=1,3,5,2,4,1,5,3,5 AND IPA=1,4,6,8,9,10. )
C
C    HMAX     IS A REAL VARIABLE THAT IS SET TO ZERO IF DERIVATIVES ARE
C             TO BE SUPPLIED ANALYTICALLY THROUGH FUNC OR TO AN UPPER
C             LIMIT FOR STEP-LENGTHS TO BE USED WHEN DIFFERENCING TO
C             APPROXIMATE DERIVATIVES.  HMAX \ 0.
C
C     IE     INTEGER VARIABLE SET TO ZERO IF THE USER WISHES TO SOLVE A
C            SPARSE SYSTEM OF NON-LINEAR EQUATIONS.
C
C            IF THE USER WISHES TO GENERATE A COLUMN OF THE VARIANCE-
C            COVARIANCE MATRIX AFTER A SUCCESSFUL CALL TO NLSPAR WITH
C            IE=0. NLSPAR IS CALLED AGAIN WITH IENTRY SET TO THE COLUMN
C            REQUIRED. ON THE NEXT EXIT, THE X ARRAY WILL THEN CONTAIN
C            THE IE-TH COLUMN OF THE VARIANCE-COVARIANCE MATRIX.
C
C
C  ERROR RETURNS
C
C    ERROR RETURNS OCCUR IF THE WORKSPACE W IS NOT LARGE ENOUGH OR IF
C    MORE THAN MAXFUN CALLS FOR FUNC ARE REQUIRED.  A MESSAGE IS OUTPUT
C    ON STREAM 6 (UNLESS LP IS CHANGED) AND THE CONDITIONS CAN BE
C    RECOGNIZED BY W(IW-1) BEING SET TO ZERO OR MAXFUN+1, RESPECTIVELY.
C
C
C  USE OF COMMON
C
C    THE SUBROUTINE CONTAINS A COMMON BLOCK CALLED SPARS OF THE FORM
C                 COMMON/SPARS/LP,EPS,ISCALE,IPRINT
C       THESE VARIABLES ARE SET TO 6,1.0E-14,0,0 RESPECTIVELY.
C       LP SPECIFIES THE UNIT NUMBER FOR PRINTING OF ANY RESULTS
C       REQUESTED BY THE IPRINT PARAMETER.  EPS IS THE RELATIVE MACHINE
C       ACCURACY (ROUND-OFF ERROR). ISCALE IS USED AS AN INTERFACE PARA-
C       -METER AND MUST BE INCLUDED IF THE COMMON BLOCK IS USED IN
C       ROUTINE FUNC, BUT SHOULD NOT BE SET.
C
C    IPRINT   IS AN INTEGER VARIABLE SET TO CONTROL THE PRINTING (APART
C             FROM DIAGNOSTICS).  IPRINT = 0  GIVES NO PRINTING AND
C             IPRINT " 0  RESULTS IN PRINTING AT THE FIRST AND LAST
C             ITERATION AND AT EVERY IABS(IPRINT)-TH ITERATION IN
C             BETWEEN.  IF IPRINT > 0  THEN THE ITERATION NUMBER, THE
C             MARQUART PARAMETER (SEE REFERENCE), THE SUM OF SQUARES OF
C             RESIDUALS S, THE NUMBER OF CALLS OF FUNC MADE SO FAR AND
C             THE EUCLIDEAN NORM (//X//**2 = SUM(X(I)**2) _ I = 1,...,N)
C             OF THE CURRENT APPROXIMATE SOLUTION, THE NORM OF THE LAST
C             CHANGE TO THE SOLUTION AND THE NORM OF THE VECTOR  V =
C             [PR(I) / PX(J)]TRANS * R (WHICH IS THE GRADIENT 1/2 * S)
C             ARE PRINTED.  IF IPRINT < 0  THEN THE CURRENT SOLUTION X,
C             THE CURRENT VECTOR R AND THE CURRENT GRADIENT VECTOR V ARE
C             ALSO PRINTED.
C
C    ALSO AVAILABLE TO THE USER IN A COMMON BLOCK CALLED NLIN ARE THE
C    FOLLOWING VARIABLES OF THE FORM
C          COMMON/NLIN/  RHO, SIG, HFAC, FAIM
C    THESE VARIABLES ARE SET TO 0.25, 0.75, 0.001 AND 0.25 RESPECTIVELY.
C    THE USER IS ADVISED THAT THESE ARE RECOMMENDED VALUES AND SHOULD
C    ONLY BE CHANGED IF THE USER IS AWARE OF ALL THE IMPLICATIONS OF
C    SUCH A CHANGE (SEE REFERENCE).
C
C
C
C  REFERENCE
C
C     FORTRAN SUBROUTINES FOR THE SOLUTION OF SPARSE SYSTEMS OF
C                      NON-LINEAR EQUATIONS
C     BY  J. K. REID.
C
C
C    ADAPTED FROM THE HARWELL ROUTINE NS03A AERE-R-7293 [REID]
C
C          DAVID  SMITH      JULY 1975
C     MODIFIED BY M.B.CARVER AND B.PURCELL           NOV.1975
C
C
C
      DIMENSION MES(3)
      REAL  STEP
      REAL  X(1), W(1), A(1), LAMDA
      INTEGER  IRN(1), IP(1), IRNA(1), IPA(1), IG1
C
C     LAMINC IS .TRUE. IF A SPECIAL STEP IS BEING MADE PRIOR TO A
C  DECREASE IN D. CHANGE IS .TRUE. IF X HAS CHANGED SINCE THE LAST CALL
C  OF JACOB.  UPDATE IS SET .TRUE. IF S HAS INCREASED SUFFICIENTLY
C  LITTLE FOR IT TO BE APPROPRIATE TO USE THE SCHUBERT UPDATING FORMULA
C  ON THE APPROXIMATE JACOBIAN. REDIFF IS .TRUE. IF ANOTHER CALL OF
C  JACOB IS REQUIRED.  ISCALE IS AN INTERFACE VARIABLE SUCH THAT THE
C  PARAMETER ADJUST IS PASSED TO JACOB.
C
      LOGICAL LAMINC,CHANGE,UPDATE,ADJUST,REDIFF
      COMMON/NLIN/  RHO, SIG, HFAC, FAIM
      COMMON/SPARS/ LP,EPS,ISCALE,IPRINT
      COMMON/NLSNL/ NC, LAMDA, IPRINTV, PRED
      EQUIVALENCE (STEP,HH,IG1)
      EXTERNAL FUNC
      DATA  ZERO/0E0/, TWO/2E0/, TEN/1E1/, ONE/1E0/, BIG/1E50/,
     ,                  P1/0.1E0/, P9/0.9E0/, P5/0.5E0/
      DATA  RHO/0.25/, SIG/0.75/, HFAC/0.001/, FAIM/0.25/
      DATA LP/6/,EPS/1.0E-14/,ISCALE/0/,IPRINT/0/
      DATA MES/1100,1101,1102/
*CALL STATF
C
C     SET INITIAL VALUES FOR VARIOUS VARIABLES.
C
      IF(IE.NE.0) GO TO 220
      DAIM=BIG
      D=BIG
      ALFA=(ONE-SIG)*P5
      REDIFF=.FALSE.
      ADJUST=.FALSE.
      DNORM=ZERO
      LAMINC=.FALSE.
      LAMDA=ZERO
      IT=0
      NFUN=0
C
C     PARTITION THE STORAGE IN W. DURING THE ITERATION W(NS) ONWARDS
C IS AVAILABLE TO THE LINEAR EQUATION SOLVER.
C
      NR=0
      NF=0
      IF(IPA(1).NE.0)NF=M+1
      NPHI=NF+M
      IF(HMAX.EQ.0.) GO TO 4
C
C      IF HMAX"0, CALL THE SPARSITY STRUCTURE IN JACOB.
C
      ISCALE=0
      IWA=IW-NPHI
      NH=NB=IG=NDC=NPHI
      DO 2 I=1,M
  2   W(NH+I)=HMAX
      CALL FUNC(M,N,X,W(NF+1),W(NPHI+1))
      CALL JACOB(M,N,IRN,IP,FUNC,X,W(NF+1),T,W(NH+1),-HMAX,W(NB+1),W(NPH
     ,I+1),IWA,W(IG+1),W(NDC+1),-1)
C
C     CONTINUE
C
  4   NB=NPHI+IP(N+1)-1
      ND=NB+M
      NDC=ND+N
      NNR=NB
      IF(IPA(1).NE.0)NNR=NDC
      NH=NDC+M
      N12=2*N+2
      IG=NH+N12
      NS=IG+N12+1
      IF(HMAX.EQ.ZERO)NS=NH
      CALL SUBNL(M,N,IRN,IP,IRNA,IPA,W,W,IW-2,W(NPHI+1),A,W(NB+1),
     ,           W(NR+1),W(NB+1),W(ND+1),DD,RJD,DG,0)
C
C     THIS CALL PERFORMS PRELIMINARY ANALYSIS OF THE SPARSITY STRUCTURE
C AND HAS THE WHOLE OF W AS WORKSPACE. FOLLOWING THIS CALL W(NC),...,
C W(IW-2) IS REGARDED AS PRIVATE WORKSPACE FOR SUBNL.
C
      IF(NS.GE.NC) GOTO 205
    5 CALL FUNC(M,N,X,W(NF+1),W(NPHI+1))
      NFUN=NFUN+1
      IF(IPA(1).EQ.0) GOTO 15
         DO 10 I=1,M
   10    W(NR+I)=W(NF+I)
         CALL NLPLIN(M,N,A,X,W(NR+1),.FALSE.,IRNA,IPA)
   15 CALL SUMSQ(W(NR+1),W(NR+1),S,M)
      IF(HMAX.EQ.ZERO) GOTO 30
C
C     SET PARAMETERS FOR JACOB CALL.
C
      IG1=0
      W(IG+1)=HH
      STEP = HFAC * HMAX
      DO 20 I=1,N12
   20 W(NH+I)=HH
   25 IF(S.LE.SAC) GOTO 30
         IF (.NOT. ADJUST)  ISCALE = 0
         IF (ADJUST)  ISCALE = 1
         CALL JACOB(M,N,IRN,IP,FUNC,X,W(NF+1),T,W(NH+1),-HMAX,W(NB+1),
     ,              W(NPHI+1),IDUM,W(IG+1),W(NDC+1),1)
         NFUN=NFUN+W(NB+1)
         CHANGE=.FALSE.
C
C     COMMENCE ITERATION BY PROVIDING PRINTING.
C
   30 IT=IT+1
      CALL SUMSQ(X,X,XNORM,N)
      XNORM= SQRT(XNORM)
      IPRINTV = 1
      IF(IPRINT.EQ.0) GOTO 50
         IF(MOD(IT-1,IABS(IPRINT)).NE.0) GOTO 50
            IPRINTV = 2
            WRITE(LP,35)IT,NFUN,LAMDA,S,XNORM,DNORM
*IF -DEF,AELIB5
   35     FORMAT(" AFTER ",I5," ITERATIONS AND ",I5," CALLS OF FUNC ",
     ,           "LAMDA IS",1PE12.4," AND THE SUM OF SQUARES OF ",
     ,           "RESIDUALS IS",E12.4/" THE NORMS OF THE CURRENT ",
     ,           "ITERATE X AND THE LAST CHANGE MADE TO IT ARE",2E12.4)
*ENDIF
*IF DEF,AELIB5
   35     FORMAT(' AFTER ',I5,' ITERATIONS AND ',I5,' CALLS OF FUNC ',
     ,           'LAMDA IS',1PE12.4,' AND THE SUM OF SQUARES OF ',
     ,           'RESIDUALS IS',E12.4/' THE NORMS OF THE CURRENT ',
     ,           'ITERATE X AND THE LAST CHANGE MADE TO IT ARE',2E12.4)
*ENDIF
            IF(IPRINT.GT.0) GOTO 50
               IPRINTV = 3
               WRITE(LP,40)(I,X(I),I=1,N)
*IF -DEF,AELIB5
   40          FORMAT(" THE CURRENT ITERATE X IS"/5(I8,1PE16.8))
*ENDIF
*IF DEF,AELIB5
   40          FORMAT(' THE CURRENT ITERATE X IS'/5(I8,1PE16.8))
*ENDIF
               WRITE(LP,45)(I,W(NR+I),I=1,M)
*IF -DEF,AELIB5
   45          FORMAT(" THE CURRENT VECTOR OF RESIDUALS IS"/ 5(I8,
     ,                1PE16.8))
*ENDIF
*IF DEF,AELIB5
   45          FORMAT(' THE CURRENT VECTOR OF RESIDUALS IS'/ 5(I8,
     ,                1PE16.8))
*ENDIF
   50 IF(S.LE.SAC) GOTO 215
      IF(NFUN.GE.MAXFUN) GOTO 195
      IF(LAMINC) GOTO 55
         CALL SUBNL(M,N,IRN,IP,IRNA,IPA,W,W,IW-2,W(NPHI+1),A,W(NB+1),
     ,              W(NR+1),W(NB+1),W(ND+1),DD,RJD,DG,1)
C
C     THIS CALL FACTORISES THE CURRENT MATRIX.
C
         CALL SUBNL(M,N,IRN,IP,IRNA,IPA,W,W,IW-2,W(NPHI+1),A,W(NB+1),
     ,              W(NR+1),W(NB+1),W(ND+1),DD,RJD,DG,2)
C
C     THIS CALL SOLVES CURRENT LINEAR PROBLEM.
C
         DNORM= SQRT(DD)
         IF(DNORM.LE.EPS*XNORM.OR.(DNORM.LE.STPMIN)) GOTO 215
         GOTO 75
   55 DO 60 I=1,M
   60 W(NDC+I)=-W(NB+I)
      IF(IPA(1) .NE. 0)  CALL NLPLIN(M,N,A,W(ND+1),W(NDC+1),.FALSE.,
     ,                               IRNA,IPA)
      CALL SUMSQ(W(NDC+1),W(NR+1),RJD,M)
      CALL SUMSQ(W(NDC+1),W(NDC+1),DJJD,M)
      U=P5
      IF(DJJD.GT.ZERO)U=RJD/DJJD
      IF( ABS(U).GT.P9)U= SIGN(P9,U)
      IF( ABS(U).LT.P1)U= SIGN(P1,U)
      DO 65 I=1,N
   65 W(ND+I)=X(I)-U*W(ND+I)
      CALL FUNC(M,N,W(ND+1),W(NDC+1),W)
      FDD=ZERO
      DO 70 I=1,M
         FDD=FDD+(U*W(NB+I)+W(NF+I)-W(NDC+I))**2
   70 W(NNR+I)=W(NDC+I)
      H=BIG
      IF(FDD.NE.ZERO)H=ALFA* ABS(U)*(ONE-U)* SQRT(DJJD/FDD)
      DAIM=DNORM*H
*IF -DEF,AELIB5
      REDIFF=CHANGE .AND. DNORM.LE.DAIM*FAIM
*ENDIF
*IF DEF,AELIB5
      REDIFF=CHANGE .AND. DNORM.LE.DAIM*FAIM
*ENDIF
      GOTO 85
C
C     EVALUATE FUNCTION AT NEW POINT
C
   75 DO 80 I=1,N
   80 W(ND +I)=W(ND+I)  +X(I)
      CALL FUNC(M,N,W(ND+1),W(NNR+1),W(NPHI+1))
   85 NFUN=NFUN+1
      IF(IPA(1).EQ.0) GOTO 95
         DO 90 I=1,M
   90    W(NB+I)=W(NNR+I)
         CALL NLPLIN(M,N,A,W(ND+1),W(NNR+1),.FALSE.,IRNA,IPA)
   95 CALL SUMSQ(W(NNR+1),W(NNR+1),SN,M)
C
C     CALCULATE NEW VALUE FOR LAMDA
C
      IF(LAMINC) GOTO 100
         RATIO=(S-SN)/PRED
         IF(RATIO.GE.ONE)RATIO=TWO-EPS-RATIO
         IF(RATIO.GT.SIG) GOTO 115
*IF -DEF,AELIB5
         IF(REDIFF.AND.ADJUST) GOTO 215
*ENDIF
*IF DEF,AELIB5
         IF(REDIFF.AND.ADJUST) GOTO 215
*ENDIF
         IF(REDIFF)ADJUST=.TRUE.
         IF(RATIO.GE.RHO) GOTO 105
         IF(DNORM.GT.D*TWO) GOTO 120
         IF(HMAX.EQ.ZERO) GOTO 110
         D=DNORM*P5
         IF(DNORM.GT.DAIM*FAIM) GOTO 120
         LAMINC=.TRUE.
         GOTO 125
  100 BETA=BIG
      DEN=SN-U*SO+(U-ONE)*S
      IF(DEN.NE.ZERO)BETA=P5*(SN-U*U*SO+(U*U-ONE)*S)/DEN
      D=AMAX1(P1,AMIN1(P5,BETA,H))*DNORM
      LAMINC=.FALSE.
      GOTO 120
  105 D=DNORM
      GOTO 125
  110 F=AMAX1(TWO,AMIN1(TEN,TWO+(S-SN)/RJD))
      D=DNORM/F
      GOTO 120
  115 D=AMIN1(D*TWO, SQRT((ONE-SIG)/(ONE-RATIO))*DNORM)
      REDIFF=.FALSE.
  120 LAMDA=AMAX1(LAMDA+(ONE-DNORM/D)*DD/DG,ZERO)
  125 UPDATE=SN.LE.S*TWO
      IF(SN.LE.S) GOTO 140
         IF(HMAX.EQ.ZERO) GOTO 5
         DO 130 I=1,N
  130    W(ND+I)=X(I)-W(ND+I)
         DO 135 I=1,M
  135    W(NB+I)=W(NB+I)-W(NF+I)
         SO=SN
         GOTO 155
C
C     STORE NEW ITERATE
C
  140 DO 145 I=1,N
         G=W(ND+I)-X(I)
         X(I)=W(ND+I)
  145 W(ND+I)=G
      CHANGE=.TRUE.
      DO 150 I=1,M
         G=W(NF+I)-W(NB+I)
         W(NF+I)=W(NB+I)
         W(NR+I)=W(NNR+I)
  150 W(NB+I)=G
      SO=S
      S=SN
      IF(HMAX.EQ.ZERO) GOTO 30
  155 IF(LAMINC) GOTO 30
      IF(REDIFF) GOTO 25
      IF(.NOT.UPDATE) GOTO 30
C
C     CORRECT THE JACOBIAN
C
      CALL NLPLIN(M,N,W(NPHI+1),W(ND+1),W(NB+1),.FALSE.,IRN,IP)
      DO 160 I=1,M
  160 W(NDC+I)=ZERO
      DO 170 J=1,N
         K1=IP(J)
         K2=IP(J+1)-1
         IF(K2.LT.K1) GOTO 170
            DO 165 K=K1,K2
               I=IRN(K)
  165       W(NDC+I)=W(NDC+I)+W(ND+J)**2
  170 CONTINUE
      DO 180 J=1,N
         K1=IP(J)
         K2=IP(J+1)-1
         IF(K2.LT.K1) GOTO 180
            DO 175 K=K1,K2
               I=IRN(K)
  175       IF(W(NDC+I).GT.ZERO)W(NPHI+K)=W(NPHI+K)-W(NB+I)*W(ND+J)
     ,                                      /W(NDC+I)
  180 CONTINUE
      GOTO 30
C
C      RETURNS
C
*IF -DEF,AELIB5
 195  CALL AELERR( MES(1),2HNF )
*ENDIF
*IF DEF,AELIB5
 195  CALL AELERR( MES(1),'NF')
*ENDIF
      NFUN=MAXFUN+1
      GO TO 215
*IF -DEF,AELIB5
 205  CALL AELERR( MES(2),2HNF )
*ENDIF
*IF DEF,AELIB5
 205  CALL AELERR( MES(2),'NF')
*ENDIF
  215 W(IW)=AMAX1(W(IW),NS+IW-NC+ONE)
      W(IW-1)=NFUN
      ADJUST=.TRUE.
      IF(IPRINTV.NE.1 .OR. NFUN.EQ.0 .OR. IPRINT.EQ.0) RETURN
      WRITE(LP,35)IT,NFUN,LAMDA,S,XNORM,DNORM
      IF(IPRINT.GT.0)RETURN
      WRITE(LP,40)(I,X(I),I=1,N)
      WRITE(LP,45)(I,W(NR+I),I=1,M)
      RETURN
C
C
C        ENTRY POINT
C          - GENERATE A COLUMN OF THE VARIANCE - COVARIANCE MATRIX
C
  220 IF(NFUN.GT.0.AND.NFUN.NE.MAXFUN+1) GOTO 230
*IF -DEF,AELIB5
      CALL AELERR(MES(3),2HNF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MES(3),'NF')
*ENDIF
         RETURN
  230 IF(LAMDA.EQ.ZERO) GOTO 235
         LAMDA=ZERO
         CALL SUBNL(M,N,IRN,IP,IRNA,IPA,W,W,IW-2,W(NPHI+1),A,W(NB+1),
     ,              W(NR+1),W(NB+1),W(ND+1),DD,RJD,DG,1)
  235 DO 240 J=1,N
  240 X(J)=ZERO
      X(IE)=ONE
      CALL SUBNL(M,N,IRN,IP,IRNA,IPA,W,W,IW-2,W(NPHI+1),A,W(NB+1),
     ,           W(NR+1),X,W(ND+1),DD,RJD,DG,3)
      RETURN
      END
      SUBROUTINE SUBNL(M,N,IRN,IP,IRNA,IPA,W,IN,IW,PHI,A,Y,R,V,S,
     ,                 DD,DV,DG,IENT)
      INTEGER  IN(1), IRN(1), IPA(1), IRNA(1), IP(1)
      REAL W(1),PHI(1),A(1),LAMDA,Y(1),R(1),S(1),V(1)
      COMMON/SPARS/  LP, EPS, IDUMMY(2)
      COMMON/NLIN/  RHO, SIG, HFAC, FAIM
      COMMON/NLSNL/ NS, LAMDA, IPRINT, PRED
      DATA  LP/6/, EPS/1.0E-14/, ZERO/0.0E0/, ONE/1.0E0/
      DATA  RHO/0.25/, SIG/0.75/, HFAC/0.001/, FAIM/0.25/
      DATA MES/1105/
C
C    THIS ENTRY PERFORMS PRELIMINARY WORK IN ANTICIPATION OF THE
C SOLUTION OF A SEQUENCE OF LINEARIZED PROBLEMS. W IS A WORKSPACE OF
C DIMENSION IW AND HAS IMPLICITLY BEEN EQUIVALENCED TO IN. FOLLOWING
C THIS ENTRY W(NS),...,W(IW) IS REGARDED AS PRIVATE WORKSPACE FOR
C IENT=1/2/3.  FAILURE (BECAUSE OF LACK OF STORAGE IS INDICATED BY
C NS=-1 AFTER A SUCCESSFUL ENTRY W(IW+2) IS SET TO INDICATE THE LEAST
C VALUE FOR IW.
C
C     FIND THE SPARSITY PATTERN OF THE JACOBIAN MATRIX USING
C  A LINKED LIST TO HOLD COLUMN NUMBERS AND ROW LINKS.  IN(NP+I),I=1,M
C  POINT TO ROW STARTS.
C
      IF(IENT.EQ.1) GOTO 110
      IF(IENT.EQ.2) GOTO 195
      IF(IENT.EQ.3) GOTO 230
      MAXINT=2**17-1
      N1=N+1
      IIN = IW
      N2=N1*2
      N6=N1*6
      NP=IIN-M
      IF(NP.LE.0) GOTO 105
      L=-1
      DO 5 I=1,M
    5 IN(NP+I)=0
      DO 30 J1=1,N
         J=N1-J1
         K1=IP(J)
         K2=IP(J+1)-1
         IF(K2.LT.K1) GOTO 15
            DO 10 K=K1,K2
               I=IRN(K)
               L=L+2
               IF(L.GE.NP) GOTO 105
               IN(L)=J
               IN(L+1)=IN(NP+I)
   10       IN(NP+I)=L
   15    IF(IPA(1).EQ.0) GOTO 30
            K1=IPA(J)
            K2=IPA(J+1)-1
            IF(K2.LT.K1) GOTO 30
               DO 25 K=K1,K2
                  I=IRNA(K)
                  IF(IN(NP+I).EQ.0) GOTO 20
                     IF(IN(IN(NP+I)).EQ.J) GOTO 25
   20             L=L+2
                  IF(L.GE.MAXINT) GOTO 95
                  IF(L.GE.NP) GOTO 105
                  IN(L)=J
                  IN(L+1)=IN(NP+I)
                  IN(NP+I)=L
   25          CONTINUE
   30 CONTINUE
C
C     USE A SIMILAR LINKED LIST STRUCTURE TO HOLD THE SPARSITY PATTERN
C OF THE UPPER-TRIANGULAR PART OF THE NORMAL MATRIX, EXCLUDING THE
C DIAGONAL.
C
      NP=IIN-N
      LC=NP-1
      DO 35 I=1,N
   35 IN(NP+I)=0
      DO 60 KI=1,L,2
         KJ=KI
         I=IN(KI)
         LI=IN(NP+I)+NP
         LL=NP+I-1
   40    KJ=IN(KJ+1)
         IF(KJ.EQ.0) GOTO 60
   45       IF(LI.EQ.NP) GOTO 55
               IF(IN(LI)-IN(KJ)) 50,40,55
   50          LL=LI
               LI=IN(LI+1)+NP
               GOTO 45
   55       IN(LL+1)=LC-NP
            IN(LC)=IN(KJ)
            IN(LC+1)=LI-NP
            LL=LC
            LC=LC-2
            IF(NP-LC.GE.MAXINT) GOTO 95
            IF(LC-L) 105,105,40
   60 CONTINUE
      LC=LC+2
      JW=L+1+MAX0(M,IIN-LC+1)
C
C     REORDER SPARSITY STRUCTURE TO FORM REQUIRED FOR PDSPARL.
C
      KN=1
      DO 70 I=1,N
         IN(I)=KN
         IN(N6+KN)=I
         KN=KN+1
         L=IN(NP+I)+NP
   65    IF(KN.GE.LC) GOTO 105
         IF(L.EQ.NP) GOTO 70
            IN(N6+KN)=IN(L)
            KN=KN+1
            L=IN(L+1)+NP
            GOTO 65
   70 CONTINUE
      IN(N1)=KN
      L2 = 3
      IA=(IIN-N6)/L2
      JW=MAX0(JW,N6+(KN+1)*L2)
      NP=IW-IA
      IF(IA.LE.KN) GOTO 105
C
C     USE PDSPARL TO ANALYSE NORMAL MATRIX USING DUMMY MATRIX ELEMENTS.
C
      DO 80 I=1,N
         K1=IN(I)
         K2=IN(I+1)
         DO 75 K=K1,K2
   75    W(NP+K)=ZERO
   80 W(NP+K1)=ONE
      CALL PDSPARL(W(NP+1),IN(N6+1),IN,N,N1,IA,B,MTYPE,1)
      IF(IN(N2).EQ.-3) GOTO 105
C
C     MOVE INTEGER ARRAYS TO END OF WORKSPACE.
C
      KA=IN(N1)-1
      JW=MAX0(JW,N6+KA*L2)
      NS = IW - 2*KA - N2 - 1
      IF(NS.LE.0) GOTO 105
         INRN=IIN-KA
         DO 85 I=1,KA
   85    IN(INRN+I)=IN(N6+I)
         INP=INRN-N2
         DO 90 I=1,N2
   90    IN(INP+I)=IN(I)
         W(IW+2) = JW +1
         RETURN
C
C     OVER-POPULATED MATRICES MAY CAUSE THE INDICES ABOVE TO EXCEED
C     2**17 WHICH WOULD REQUIRE MORE THAN 18 BIT ADDRESS REFERENCING.
C     THIS IS NOT POSSIBLE IN CURRENT FORTRAN
C
*IF -DEF,AELIB5
  95  CALL AELERR( MES,2HNF )
*ENDIF
*IF DEF,AELIB5
  95  CALL AELERR( MES,'NF')
*ENDIF
  105 NS=-1
      RETURN
C
C
C        ENTRY POINT
C     FORM AND FACTORIZE LEAST SQUARES MATRIX. WORKSPACE Y HAS LENGTH M.
C
  110 EP = EPS
  115 DO 120 I=1,M
  120 Y(I)=ZERO
      SSC=ZERO
      DO 180 II=1,N
         I=IN(INP+N1+II)
         KP1=IP(I)
         KP2=IP(I+1)-1
         IF(KP2.LT.KP1) GOTO 130
            DO 125 K=KP1,KP2
  125       Y(IRN(K))=PHI(K)
  130    IF(IPA(1).EQ.0) GOTO 140
            KA1=IPA(I)
            KA2=IPA(I+1)-1
            IF(KA2.LT.KA1) GOTO 140
               DO 135 K=KA1,KA2
  135          Y(IRNA(K))=Y(IRNA(K))+A(K)
  140    J1=IN(INP+I)
         J2=IN(INP+IN(INP+N1+II+1))-1
         DO 160 KK=J1,J2
            J=IN(INRN+KK)
            PROD=ZERO
            K1=IP(J)
            K2=IP(J+1)-1
            IF(K2.LT.K1) GOTO 150
               DO 145 K=K1,K2
  145          PROD=PROD+PHI(K)*Y(IRN(K))
  150       IF(IPA(1).EQ.0) GOTO 160
               K1=IPA(J)
               K2=IPA(J+1)-1
               IF(K2.LT.K1) GOTO 160
                  DO 155 K=K1,K2
  155             PROD=PROD+A(K)*Y(IRNA(K))
  160    W(NS+KK)=PROD
         SSC=AMAX1(SSC,W(NS+J1))
         IF(KP2.LT.KP1) GOTO 170
            DO 165 K=KP1,KP2
  165       Y(IRN(K))=ZERO
  170    IF(IPA(1).EQ.0) GOTO 180
            IF(KA2.LT.KA1) GOTO 180
               DO 175 K=KA1,KA2
  175          Y(IRNA(K))=ZERO
  180 CONTINUE
C
C     ADD MULTIPLE OF IDENTITY MATRIX
C
      SSC=AMAX1(SSC*EP ,LAMDA)
      DO 185 I=1,N
         J=NS+IN(INP+I)
  185 W(J)=W(J)+SSC
C
C     FACTORIZE NORMAL MATRIX
C
      CALL PDSPARL(W(NS+1),IN(INRN+1),IN(INP+1),N,N1,IA,B,MTYPE,3)
      J=-IN(INP+2*N1)
      IF(J.NE.4 .AND. J.NE.5)RETURN
      EP=EP+EP
      WRITE(LP,190)
*IF -DEF,AELIB5
  190 FORMAT(" PDSPARL ERROR IS BEING HANDLED BY NLSPAR")
*ENDIF
*IF DEF,AELIB5
  190 FORMAT(' PDSPARL ERROR IS BEING HANDLED BY NLSPAR')
*ENDIF
      GOTO 115
C
C
C        ENTRY POINT
C    SOLVE SYSTEM, ASSUMING FACTORIZATION KNOWN
C
  195 DO 200 I=1,N
  200 V(I)=ZERO
      CALL NLPLIN(M,N,PHI,R,V,.TRUE.,IRN,IP)
      IF (IPA(1) .NE. 0)  CALL NLPLIN(M,N,A,R,V,.TRUE.,IRNA,IPA)
      DO 205 I=1,N
  205 S(I)=-V(I)
C
C     PRINT VECTOR V IF REQUESTED.
C
      IF(IPRINT.EQ.1) GOTO 220
         CALL SUMSQ(S,S,SS,N)
         SS= SQRT(SS)
         WRITE(LP,210)SS
*IF -DEF,AELIB5
  210    FORMAT(" THE NORM OF THE VECTOR V IS",1PE12.4)
*ENDIF
*IF DEF,AELIB5
  210    FORMAT(' THE NORM OF THE VECTOR V IS',1PE12.4)
*ENDIF
         IF(IPRINT.EQ.2) GOTO 220
            WRITE(LP,215)(I,V(I),I=1,N)
*IF -DEF,AELIB5
  215       FORMAT(" THE CURRENT VECTOR V IS"/5(I8,1PE16.8))
*ENDIF
*IF DEF,AELIB5
  215       FORMAT(' THE CURRENT VECTOR V IS'/5(I8,1PE16.8))
*ENDIF
  220 CALL PDSPARL(W(NS+1),IN(INRN+1),IN(INP+1),N,N1,IA,S,1,2)
      CALL SUMSQ(S,S,DD,N)
      CALL SUMSQ(S,V,DV,N)
      DO 225 I=1,N
  225 V(I)=-S(I)
      CALL PDSPARL(W(NS+1),IN(INRN+1),IN(INP+1),N,N1,IA,V,1,2)
      CALL SUMSQ(S,V,DG,N)
      PRED=LAMDA*DD-DV
      RETURN
C
C
C        ENTRY POINT
C
  230 CALL PDSPARL(W(NS+1),IN(INRN+1),IN(INP+1),N,N1,IA,V,1,2)
      RETURN
      END
      SUBROUTINE NLPLIN(M,N,A,X,AX,TRANS,IRN,IP)
C
C THIS SUBROUTINE CALCULATES AX+AT*X IF TRANS IS .TRUE. (AX +A*X IF
C TRANS IS .FALSE.)
C AND RETURNS THE RESULT IN AX
C
C A IS A SPARSE MATRIX STORED BY COLUMNS WHOSE SPARSITY STRUCTURE
C IS DESCRIBED BY IRN AND IP
C
C
C
      INTEGER   IRN(1),IP(1)
      DIMENSION A(1),X(1),AX(1)
      LOGICAL TRANS
      K=M
      IF(TRANS)K=N
      DO 20 J=1,N
         K1=IP(J)
         K2=IP(J+1)-1
         IF(K1.GT.K2) GOTO 20
            IF(TRANS) GOTO 10
               DO 5 K=K1,K2
                  I=IRN(K)
    5          AX(I)=AX(I)+A(K)*X(J)
               GOTO 20
   10       DO 15 K=K1,K2
               I=IRN(K)
   15       AX(J)=AX(J)+A(K)*X(I)
   20 CONTINUE
      RETURN
      END
      SUBROUTINE SUMSQ(A,B,S,N)
C
C THIS SUBROUTINE CALCULATES THE INNER PRODUCT OF VECTORS A AND B
C AND RETURNS RESULT IN S
C
C
      DIMENSION A(1),B(1)
      S=0
      DO 5 I=1,N
    5 S=S+A(I)*B(I)
      RETURN
      END
