*DECK PDSPARL
      SUBROUTINE PDSPARL(A,IND,IW,N,NP,IA,B,MTYPE,IENT)
C
C
C  PURPOSE
C
C    THIS ROUTINE SOLVES A SYMMETRIC POSITIVE-DEFINITE SPARSE SYSTEM OF
C    LINEAR EQUATIONS
C
C        SUM(A(I,J) * X(J)) _ J=1,2,...,N = B(I) ;  FOR I = 1,2,...,N
C
C    (I. E. FIND X = A**-1 * B) OR RELATED PROBLEMS.
C
C    THERE ARE THREE ENTRIES
C    1.  DECOMPOSE A INTO LDL**T, WHERE L IS LOWER TRIANGULAR AND D IS
C        DIAGONAL, USING A PIVOTAL STRATEGY DESIGNED TO MAINTAIN
C        SPARSITY.
C    2.  USE THE FACTORS PRODUCED BY 1 (OR 3) TO FIND A**-1 * B OR A*B.
C    3.  FACTORISE A NEW MATRIX A OF THE SAME PATTERN USING THE PIVOTAL
C        SEQUENCE DETERMINED BY AN EARLIER ENTRY TO 1.
C
C
C  ARGUMENT LIST
C
C    CALL PDSPARL(A,IND,IW,N,NP,IA,B,MTYPE,IENT)
C
C    A        IS A REAL ARRAY OF DIMENSION IA HOLDING THE NON-ZERO
C             ELEMENTS TO THE LOWER-TRIANGULAR PART OF THE MATRIX A ON
C             ENTRY TO IENT=1 OR 3 AND THE ELEMENTS OF D AND L ON EXIT.
C             (E. G.  A(1,1),A(3,1),A(5,1),A(2,2),A(4,2),A(3,3),A(5,3),
C             A(4,4),A(5,5)).  BEFORE ENTRY TO IENT=3, THE ELEMENTS OF A
C             SHOULD BE SET IN A DIFFERENT ORDER (SEE SECTION ON THE
C             USE OF ENTRIES).
C
C    IND      IS AN INTEGER ARRAY OF DIMENSION 2*IA, WHOSE FIRST IA
C             ELEMENTS ARE USED TO HOLD ROW NUMBERS AND WHOSE REMAINING
C             ELEMENTS ARE USED FOR WORKSPACE BY ENTRY IENT=1 ONLY. IF
C             A(I,J) IS HELD IN A(K), THEN IND(K) MUST CONTAIN I; IN THE
C             ABOVE EXAMPLE IND = 1,3,5,2,4,3,5,4,5.  IND IS ALTERED BY
C             IENT=1.
C
C    IW       IS AN INTEGER ARRAY OF DIMENSION (NP,6) WHERE NP > N+1.
C             BEFORE ENTRY TO IENT=1 THE VALUES OF IW(J,1), J=1,...,N
C             SHOULD BE SET TO THE SUBSCRIPT IN ARRAY A OF THE J-TH
C             DIAGONAL ELEMENT, AND IW(N+1,1) SHOULD BE SET TO THE
C             SUBSCRIPT OF THE FIRST UNUSED LOCATION IN A; IN THE ABOVE
C             EXAMPLE IW = 1,4,6,8,9,10.  ((IW(I,J), I=1,N+1), J=1,2)
C             SHOULD BE LEFT UNDISTURBED BETWEEN A IENT=1 ENTRY AND A
C             SUBSEQUENT ENTRY TO IENT=2 OR 3.  THE REST OF IW IS
C             AVAILABLE AS WORKSPACE.
C
C    N        IS AN INTEGER VARIABLE SET TO THE ORDER N OF THE MATRIX A.
C
C    NP       IS AN INTEGER VARIABLE SET TO THE FIRST DIMENSION OF THE
C             ARRAY IW.  NP > N+1.
C
C    IA       IS AN INTEGER VARIABLE SET TO INDICATE THE SIZE OF ARRAYS
C             A AND IND.
C
C    B        IS A REAL ARRAY OF DIMENSION N USED TO HOLD B ON ENTRY
C             AND A**-1 * B OR A*B ON EXIT FROM IENT=2 ONLY.
C
C    MTYPE    IS AN INTEGER VARIABLE CONTOLLING THE ACTION OF IENT=2.
C             IT SHOULD HAVE THE VALUE 1 OR 2 ACCORDING TO WHETHER
C             A**-1 * B OR A*B IS REQUIRED.
C
C    IENT     IS AN INTEGER VARIABLE USED TO CONTROL ENTRY TO PDSPARL.
C             IENT=1  DECOMPOSES A INTO LDL**T.
C             IENT=2  COMPUTES A**-1 * B OR A*B.
C             IENT=3  FACTORIZES A NEW MATRIX A.
C
C
C  USE OF ENTRIES
C
C    BETWEEN A CALL OF IENT=1 AND A SUBSEQUENT CALL OF IENT=2 OR IENT=3
C    THE CONTENTS OF A, (IND(I), I=1,IA), ((IW(I,J), I=1,N+1), J=1,2),
C    N, NP, IA SHOULD NOT BE ALTERED.  IW(N+1,2) IS SET TO N+1 ON
C    SUCCESSFUL COMPLETION OF IENT=1, OR NEGATIVE IF AN ERROR HAS BEEN
C    DETECTED.
C
C    IENT=3 REQUIRES THE MATRIX TO BE LOADED IN A DIFFERENT WAY SINCE IT
C    USES THE LOWER TRIANGULAR PATTERN OF THE REORDERED MATRIX WITH
C    PIVOTS IN SEQUENCE AND INCLUDING ADDITIONAL ELEMENTS CREATED DURING
C    THE ELIMINATION.  THE I-TH COLUMN IN THIS NEW ORDER WAS COLUMN
C    IW(I,2) IN THE ORIGINAL MATRIX; IT NOW STARTS AT A(IW(IW(I,2),1))
C    AND ENDS AT A(IW(IW(I+1,2),1)-1).  THE ELEMENT STORED IN A(K) IS IN
C    THE IND(K)-TH ROW OF THE ORIGINAL MATRIX.  ALL ELEMENTS A(K),
C    K = 1,2,...,KA WHERE KA = IW(N+1,1) - 1  MUST BE SET.
C
C
C
C  REFERENCE
C
C     TWO FORTRAN SUBROUTINE FOR DIRECT SOLUTION OF LINEAR EQUATIONS
C        WHOSE MATRIX IS SPARSE, SYMMETRIC AND POSITIVE-DEFINITE
C     BY  J. K. REID.
C
C
C    ADAPTED FROM THE HARWELL ROUTINE MA17A/B/C  AERE-R-7119 [REID]
C
C          DAVID  SMITH      JULY 1975
C
C
      INTEGER   IND(1),IW(NP,6)
      DIMENSION A(1),B(1)
      DIMENSION MES(8)
*CALL ERRORF
      DATA MES/1110,1111,1112,1113,1114,1115,1116,1117/
      DATA  ZERO/0.0/, LP/6/
*CALL STATF
C
C     MATRIX ELEMENTS ARE HELD IN A(K),K=1,2,...,KA.
C     ON ENTRY IND(K),K=1,KA HOLDS THE ROW NUMBER OF THE ELEMENT HELD IN
C A(K).  IN THE MAIN BODY OF THE SUBROUTINE IND(K),IND(K+IA) HOLD THE
C ADDRESS OF THE NEXT ELEMENT IN THE ROW/COLUMN IF THERE IS ONE AND
C FOR THE LAST ELEMENT IN THE ROW/COLUMN HOLD (IA+ THE ROW/COLUMN
C NUMBER).  FINALLY IND(K) IS RESET TO THE ROW NUMBER OF THE ELEMENT
C HELD IN A(K).
C     ON ENTRY AND ON EXIT IW(I,1) CONTAINS THE ADDRESS OF THE FIRST
C ELEMENT OF COLUMN I AND IW(N+1,1) CONTAINS THE ADDRESS OF THE FIRST
C UNUSED ELEMENT IN A.  ON EXIT IW(I,2) HOLDS THE COLUMN NUMBERS IN
C PIVOTAL ORDER. AFTER A SUCCESSFUL ENTRY IW(N+1,2)=N+1 AND AFTER AN
C UNSUCCESSFUL ENTRY IW(N+1,2)=-(ERROR NUMBER). IN THE MAIN BODY OF THE
C SUBROUTINE IW(I,1),IW(I,2) HOLD THE ADDRESS OF THE FIRST ELEMENT OF
C THE I TH ROW/COLUMN.
C
      IF(IENT.EQ.2) GOTO 215
      IF(IENT.EQ.3) GOTO 180
*IF -DEF,AELIB5
      IF(IENT.NE.1) CALL AELERR(MES(8),2HNF)
*ENDIF
*IF DEF,AELIB5
      IF(IENT.NE.1) CALL AELERR(MES(8),'NF')
*ENDIF
      KA=IW(N+1,1)-1
C
C     IW(I,3) HOLDS THE POSITION IN THE ORDERING BY NUMBER OF
C NON-ZEROS OF THE LAST ROW/COLUMN TO HAVE LESS THAN I NON-ZERO
C ELEMENTS OR ZERO IF NONE HAVE LESS THAN I NON-ZERO ELEMENTS.
C ON EXIT IW(I,3) HOLDS THE POSITION OF THE I TH ROW IN THE PIVOTAL
C ORDERING.
C     IW(I,4) HOLDS THE NUMBER OF NON-ZEROS IN THE I TH ROW/COLUMN.
C     IW(I,5) HOLDS THE POSITION OF THE I TH ROW/COLUMN
C IN THE ORDERING BY NUMBER OF NON-ZEROS.
C     IW(I,6) HOLDS ROW/COLUMN NUMBERS IN PIVOTAL ORDER FOR
C I@IP AND IN ORDER OF INCREASING NUMBERS OF NON-ZEROS OTHERWISE.
C
C
      N1=N+1
      DO 10 I=1,N1
         IW(I,2)=IW(I,1)
         IW(I, 6)=I
         DO 10 J=3,5
   10 IW(I,J)=0
C
C      SET ROW AND COLUMN LINKS, FIND FIRST ELEMENTS
C OF THE ROWS AND COUNT THE NUMBER OF NON-ZEROS IN THE ROWS.
C
      DO 20 J=1,N
C
C     TEMPORARILY WE USE IW(I,5) TO HOLD THE ADDRESS OF THE LAST NON-
C ZERO ENCOUNTERED IN THE I TH ROW.
C
         K1=IW(J,2)
         K2=IW(J+1,2)-1
C
C     CHECK THAT ALL DIAGONAL ELEMENTS ARE PRESENT.
C
         IF(IND(K1).NE.J) GOTO 310
         IL=0
         DO 15 K=K1,K2
            I=IND(K)
            IF(I.LE.IL) GOTO 280
            IL=I
            IW(I,4)=IW(I,4)+1
            IF(I.NE.J)IW(J,4)=IW(J,4)+1
            IND(K)=I+IA
            IND(K+IA)=K+1
            KL=IW(I,5)
            IF(KL.LE.0)IW(I,1)=K
            IF(KL.GT.0)IND(KL)=K
   15    IW(I,5)=K
   20 IND(K2+IA)=IA+J
C
C     SEND IW(*,5) TO BE SORTED AS SORTAG REARRANGES ,
C          LEAVING IW(*,4) UNCHANGED
C
      DO 25 I=1,N
         IW(I,5) = IW(I,4)
   25 CONTINUE
C
C     SET UP THOSE VECTORS IN IW ASSOCIATED WITH ORDERING BY NUMBERS
C OF NON-ZEROS.
C
      CALL SORTAG(IW(1,5), 1, N, IW(1,6))
      DO 30 I=1,N
         J=IW(I,6)
         IW(J,5)=I
         NZ=IW(J,4)
   30 IW(NZ+1,3)=I
      J=0
      DO 35 I=1,N
         IF(IW(I,3).EQ.0)IW(I,3)=J
   35 J=IW(I,3)
C
C     NOW PERFORM THE MAIN ELIMINATION.
C
      DO 140 IP=1,N
      JP=IW(IP,6)
C
C     JP IS THE PIVOTAL ROW/COLUMN NUMBER.
C
      KP=IW(JP,2)
C     KP IS THE ADDRESS OF THE PIVOT.
C
C     MOVE THE PIVOTAL ROW/COL TO FRONT AND REPLACE ROW LINKS BY ROW
C  NUMBERS.
C
      K=IW(JP,1)
C     K IS USED TO SCAN THE PIVOTAL ROW/COLUMN.
      LL=1
C
C     LL=1,2 ACCORDING AS THE ROW/COLUMN LINKS ARE NEEDED TO SCAN THE
C PIVOTAL ROW.
C
   40 M=K
      LIA=IA*(2-LL)
   45 M=IND(M+LIA)
      IF(M.LE.IA) GOTO 45
      L=M-IA
      IF(L.EQ.JP)LL=2
C
C     ON THE REMOVAL OF THE ELEMENT A(L,JP) THE FOLLOWING
C INSTRUCTIONS ARE USED TO UPDATE THE NUMBERS OF ELEMENTS IN THE
C CORRESPONDING ROW AND COLUMN AND MAKE CONSEQUENT CHANGES TO THE
C ORDERING BY NUMBER OF NON-ZEROS.
C
      IR=L
      DO 55 LM=1,2
         NZ=IW(IR,4)-1
         IW(IR,4)=NZ
         JPOS=IW(NZ+1,3)+1
         IPOS=IW(IR,5)
         IF(IPOS.EQ.JPOS) GOTO 50
            JJ=IW(IPOS,6)
            JR=IW(JPOS,6)
            IW(IPOS,6)=IW(JPOS,6)
            IW(JPOS,6)=JJ
            JJ=IW(IR,5)
            IW(IR,5)=IW(JR,5)
            IW(JR,5)=JJ
   50    IW(NZ+1,3)=JPOS
         IF(L.EQ.JP) GOTO 70
   55 IR=JP
C
C     NOW MAKE LINKS SKIP OVER PIVOTAL ROW/COLUMN.
C
      KN=IW(L,3-LL)
      IF(KN.NE.K) GOTO 60
         IW(L,3-LL)=IND(K+LIA)
         GOTO 65
   60 KL=KN
      KN=IND(KL+LIA)
      IF(KN.NE.K) GOTO 60
      IND(KL+ LIA)=IND(K+ LIA)
   65 IND(K+ IA)=IND(K+IA-LIA)
   70 IND(K)=L
      K=IND(K+IA)
      IF(K.LE.IA) GOTO 40
C
C     NOW LOOK FOR FILL-IN.
C
      K=IW(JP,1)
   75 L=IND(K)
      IF(L.EQ.JP) GOTO 135
      KI=IW(JP,1)
      KL=IW(L,1)
   80 JI=IND(KI)
      IF(JI.EQ.JP) GOTO 130
   85 M=KL
   90 M=IND(M+IA)
      IF(M.LE.IA) GOTO 90
      JL=M-IA
      IF(JI-JL) 95,130,125
C
C     CREATE A NEW NON-ZERO IN POSITION (L,JI)
C
   95 KA=KA+1
      IF(KA.GT.IA) GOTO 300
      A(KA)=ZERO
      IR=JI
      IC=L
      DO 120 LM=1,2
         KM=IW(IC,LM)
         KLAST=0
         LIA=(LM-1)*IA
  100    M=KM
  105    M=IND(M+IA-LIA)
         IF(M.LE.IA) GOTO 105
         IF(M-IA.GT.IR) GOTO 110
            KLAST=KM
            KM=IND(KM+LIA)
            IF(KM.LE.IA) GOTO 100
  110    IND(KA+LIA)=KM
         IF(KLAST.NE.0)IND(KLAST+LIA)=KA
         IF(KLAST.EQ.0)IW(IC,LM)=KA
         NZ=IW(IR,4)
         IW(IR,4)=NZ+1
         JPOS=IW(NZ+1,3)
         JR=IW(JPOS,6)
         IF(IR.EQ.JR) GOTO 115
            IPOS=IW(IR,5)
            JJ=IW(IPOS,6)
            IW(IPOS,6)=IW(JPOS,6)
            IW(JPOS,6)=JJ
            JJ=IW(IR,5)
            IW(IR,5)=IW(JR,5)
            IW(JR,5)=JJ
  115    IW(NZ+1,3)=JPOS-1
         IR=IC
  120 IC=JI
      KL=KA
      GOTO 130
  125 KL=IND(KL)
      IF(KL-IA) 85,85,135
  130 KI=IND(KI+IA)
      IF(KI.LE.IA) GOTO 80
  135 K=IND(K+IA)
      IF(K.LE.IA) GOTO 75
  140 CONTINUE
C
C     SCAN THE MATRIX SETTING UP ORDERING NUMBERS IN IND(I+IA) AND
C POINTERS TO PIVOTS IN REORDERED MATRIX IN IW(I,4).
C
      J=1
      DO 150 I=1,N
         IP=IW(I,6)
         IW(IP,4)=J
         K=IW(IP,1)
  145    KN=IND(K+IA)
         IND(K+IA)=J
         J=J+1
         K=KN
         IF(K.LE.IA) GOTO 145
  150 CONTINUE
      IW(N1,4)=J
C
C     REORDER.
C
      KA=J-1
      DO 160 I=1,KA
         IF(I.EQ.IND(I+IA)) GOTO 160
            A1=A(I)
            I1=IND(I)
            J=I
  155       K=IND(J+IA)
            IND(J+IA)=J
            A2=A(K)
            I2=IND(K)
            A(K)=A1
            IND(K)=I1
            A1=A2
            I1=I2
            J=K
            IF(K.NE.I) GOTO 155
  160 CONTINUE
C
C     SET VECTORS IN PREPARATION FOR FACTOR AND OPERATE.
C
      DO 165 I=1,N1
         IW(I,1)=IW(I,4)
  165 IW(I,2)=IW(I,6)
C
C     REORDER THE COLUMN ELEMENTS INTO PIVOTAL ORDER.
C
      DO 175 IP=1,N
         IC=IW(IP,2)
         K=IW(IC,1)
         K1=K+1
         K2=IW(IW(IP+1,2),1)-1
  170    K=K+1
         IF(K.GT.K2) GOTO 175
            IF(IW(IND(K),5).GE.IW(IND(K-1),5)) GOTO 170
            AM=A(K)
            A(K)=A(K-1)
            A(K-1)=AM
            I=IND(K)
            IND(K)=IND(K-1)
            IND(K-1)=I
            IF(K.NE.K1)K=K-2
            GOTO 170
  175 CONTINUE
C
C
C     FACTOR ENTRY
C
  180 N1 = N + 1
      IF(IW(N1,2).EQ.-5 .OR. IW(N1,2).EQ.-4)IW(N1,2)=N1
      IF(IW(N1,2).NE.N1) GOTO 335
      IFL=0
      DO 210 IP=1,N
         IR=IW(IP,2)
         KP=IW(IR,1)
         K2=IW(IW(IP+1,2),1)-1
         IF(A(KP)) 185,290,190
  185    IFL=IR
  190    K1=KP+1
         IF(K1.GT.K2) GOTO 210
            DO 205 K=K1,K2
               AL=A(K)/A(KP)
               IC=IND(K)
               L=IW(IC,1)-1
               DO 200 M=K,K2
  195             L=L+1
                  IF(IND(M).NE.IND(L)) GOTO 195
  200          A(L)=A(L)-AL*A(M)
  205       A(K)=AL
  210 CONTINUE
      IF(IFL.GT.0) GOTO 320
      RETURN
C
C
C     OPERATE ENTRY
C
  215 N1 = N + 1
      IF(IW(N1,2).NE.N1.AND.IW(N1,2).NE.-5) GOTO 335
      IWN1=IW(N1,2)
      IW(N1,2)=N1
C
C     FORWARD SUBSTITUTION.
C
      DO 245 IP=1,N
         IC=IW(IP,2)
         KP=IW(IC,1)
         K1=KP+1
         K2=IW(IW(IP+1,2),1)-1
         IF(MTYPE.EQ.2) GOTO 230
            IF(K1.GT.K2) GOTO 225
               DO 220 K=K1,K2
                  IR=IND(K)
  220          B(IR)=B(IR)-A(K)*B(IC)
  225       B(IC)=B(IC)/A(KP)
            GOTO 245
  230    IF(K1.GT.K2) GOTO 240
            DO 235 K=K1,K2
  235       B(IC)=B(IC)+A(K)*B(IND(K))
  240    B(IC)=B(IC)*A(KP)
  245 CONTINUE
C
C     BACKWARD SUBSTITUTION
C
      DO 265 II=1,N
         IP=1+N-II
         IR=IW(IP,2)
         K1=IW(IR,1)+1
         K2=IW(IW(IP+1,2),1)-1
         IF(K1.GT.K2) GOTO 265
            IF(MTYPE.EQ.2) GOTO 255
               DO 250 K=K1,K2
  250          B(IR)=B(IR)-A(K)*B(IND(K))
               GOTO 265
  255       DO 260 K=K1,K2
               IC=IND(K)
  260       B(IC)=B(IC)+A(K)*B(IR)
  265 CONTINUE
      IW(N1,2)=IWN1
      IF(IW(N1,2).EQ.-5) GOTO 345
      RETURN
C
C     THE FOLLOWING INSTRUCTIONS IMPLEMENT THE FAILURE EXITS.
C
 280  ENCODE(5,1,IERWD(3))K
      IERWD(3)=IERWD(3).AND.MASK(30)
  1   FORMAT(I5)
*IF -DEF,AELIB5
      CALL AELERR( MES(1),2HNF )
*ENDIF
*IF DEF,AELIB5
      CALL AELERR( MES(1),'NF')
*ENDIF
      IW(N1,2)=-1
      RETURN
 290  ENCODE(5,1,IERWD(3))IR
      IERWD(3)=IERWD(3).AND.MASK(30)
*IF -DEF,AELIB5
      CALL AELERR( MES(2),2HNF )
*ENDIF
*IF DEF,AELIB5
      CALL AELERR( MES(2),'NF')
*ENDIF
      IW(N1,2)=-4
      RETURN
 300  ENCODE(5,1,IERWD(3))IP
      IERWD(3)=IERWD(3).AND.MASK(30)
*IF -DEF,AELIB5
      CALL AELERR( MES(3),2HNF )
*ENDIF
*IF DEF,AELIB5
      CALL AELERR( MES(3),'NF')
*ENDIF
      IW(N1,2)=-3
      RETURN
 310  ENCODE(5,1,IERWD(3))J
      IERWD(3)=IERWD(3).AND.MASK(30)
*IF -DEF,AELIB5
      CALL AELERR( MES(4),2HNF )
*ENDIF
*IF DEF,AELIB5
      CALL AELERR( MES(4),'NF')
*ENDIF
      IW(N1,2)=-2
      RETURN
 320  ENCODE(5,1,IERWD(3))IFL
      IERWD(3)=IERWD(3).AND.MASK(30)
*IF -DEF,AELIB5
      CALL AELERR( MES(5),2HNF )
*ENDIF
*IF DEF,AELIB5
      CALL AELERR( MES(5),'NF')
*ENDIF
      IW(N1,2)=-5
      RETURN
*IF -DEF,AELIB5
 335  CALL AELERR(MES(6),2HNF)
*ENDIF
*IF DEF,AELIB5
 335  CALL AELERR(MES(6),'NF')
*ENDIF
      RETURN
*IF -DEF,AELIB5
 345  CALL AELERR(MES(7),2HNF)
*ENDIF
*IF DEF,AELIB5
 345  CALL AELERR(MES(7),'NF')
*ENDIF
      RETURN
      END
