*DECK RKFINT
      SUBROUTINE RKFINT(RKGET,Y,N,T,HINTVL,EMAX,DT,MF,INOUT,WS)
C
C PURPOSE
C   INTEGRATION OF A SYSTEM OF N NONLINEAR FIRST ORDER
C   DIFFERENTIAL EQUATIONS.
C
C INTRODUCTION
C   THIS ROUTINE USES THE RUNGE KUTTA METHOD  AND FEHLBERG"S FORMULA
C   NO. 2. STEP SIZE IS ADJUSTED THROUGH THE ESTIMATE FOR LOCAL ERROR.
C
C REMARKS - FROM NOW ON, L.E. IS USED AS THE SHORT FORM FOR LOCAL ERROR.
C           ONLY RELATIVE CRITERION IS AVAILABLE IN CONTROLLING THE L.E.
C           NOTE THAT TO PREVENT PROBLEMS ARISING FROM VALUE OF Y
C           VANISHING, THERE IS A LOWER BOUND FOR VALUE OF Y USED IN
C           CALCULATING L.E. AND IT IS CHOSEN BY THE USER. (DEFAULT
C           VALUE IS 1.E-14)
C
C DESCRIPTION OF PARAMETERS
C
C  INPUT VARIABLE
C  RKGET - EXTERNAL FUNCTION SUPPLIED BY USER IN FORM OF
C            SUBROUTINE RKGET(N,T,Y,DY) WHICH WILL EVALUATE THE
C             RIGHT HAND SIDE OF THE DIFFERENTIAL EQUATIONS
C   Y     - REAL N-VECTOR CONTAINING THE INITIAL VALUES OF THE
C             DEPENDENT VARIABLES.
C   N     - NUMBER OF FIRST ORDER DIFFERENTIAL EQUATIONS
C   T     - REAL VARIABLE, IT SHOULD CONTAIN THE STARTING VALUE OF
C             THE INDEPENDENT VARIABLE T. DEFAULT IS 0.
C   HINTVL- LENGTH OF INTERVAL TO BE INTEGRATED. HINTVL.EQ.0. IMPLIES
C             CONTROL WILL RETURN TO USER AFTER EACH SUCCESSFUL STEP.
C             IF WANT TO CHANGE HINTVL AFTERWARDS, IFLAG HAS TO BE RESET
C             TO MAKE AN INITIALIZATION CALL.
C             IF UNINITIALIZED IT IS ASSUMED TO BE 0.
C   EMAX  - EMAX.EQ.0 IMPLIES FIXED STEP SIZE IS USED. OTHERWISE IT IS
C             TREATED AS THE RELATIVE TOLERANCE IN YBASE, AND RKF45
C             ADJUSTS STEP SIZE SUCH THAT THE ESTIMATE OF L.E. <=
C             EMAX*YBASE. YBASE IS Y(I) FOR Y(I) .GT. YCUT. OTHERWISE
C             YBASE IS YCUT. DEFAULT VALUE FOR EMAX IS 1.E-5
C   DT    - REAL VARIABLE CONTAINING AN ESTIMATE OF INITIAL STEP SIZE.
C             UNINITIALIZED ON INPUT IMPLIES THAT IT IS GOING
C             TO BE DETERMINED BY THE SUBROUTINE.(IF EMAX=0. THEN FIXED
C             STEP SIZE WILL BE TAKEN, WHICH IS THE SIZE OF THE FIRST
C             STEP WHETHER IT IS SUPPLIED OR CALCULATED BY THE ROUTINE.)
C  MF    A NON ZERO VALUE SUPPRESSES NON FATAL DIAGNOSTIC MESSAGES.
C  INOUT OPERATOR FLAG WHICH MUST BE SET = 0 FOR THE FIRST CALL
C        RETURNS A VALUE OF PLUS 1 WHEN INTEGRATION IS SUCCESSFUL
C        A NEGATIVE VALUE INDICATES AN AN ERROR FLAG.
C   WS    - REAL ARRAY OF USER SUPPLIED WORKING STORAGE. SHOULD BE OF
C             DIMENSION AT LEAST 8*N
C
C  OUTPUT VARIABLE
C  INOUT - ERROR FLAG TO INDICATE TYPE OF ERROR ENCOUNTERED
C            =-1 A NUMBER OF STEPS KOUNT(3) WERE TAKEN AT DTMAX
C                SO EFFICIENCY WAS DEGRADED
C            =-3 DTUSED WAS SO SMALL IN RELATION TO Y.
C                ACCURACY MAY HAVE BEEN DEGRATED FOR KOUNT(5) STEPS.
C            =-4 ERROR CRITERION TOO STRINGENT AS 10 CONSECUTIVE
C                STEPS HAVE FAILED THE ABOVE TWO CRITERIA.
C            =-5 ILLEGAL VALUE FOR N,EPS,DTMIN OR DTMAX
C             NOTE THAT DTMAX IS HIT IMPLIES THAT DTMAX SHOULD
C             POSSIBLY BE INCREASED WHILE DTMIN IS HIT IMPLIES THAT THE
C             RESULT MAY NOT FULFILL THE REQUIRED ACCURACY AND IT SHOULD
C             POSSIBLY BE REDUCED. (WHEN DT CORRESPONDING TO DTMIN IS
C             USED THE ERROR CRITERION IS NOT FORCED TO BE FULFILLED)
C   T     - FINAL VALUE OF T REACHED, I.E., (T OF INPUT + HINTVL)
C   Y     - VALUE OF THE VECTOR Y AT T
C   DT    - LAST NATURAL STEP SIZE PERMITTED BY THE ALGORITHM.
C   WS    - FIRST N ENTRIES OF WS CONTAINS THE MAX. L.E. IN EACH
C             COMPONENT OF Y ENCOUNTERED DURING THE INTEGRATION. IT IS
C             INITIALIZED TO 0. ONLY ON FIRST CALL OF A PROBLEM
C
C              IN      / STAT /
C  KOUNT(1) - TOTAL NUMBER OF CALLS TO AUXILIARY SUBROUTINE
C  KOUNT(2) NUMBER OF SUCCESSFUL STEPS TAKEN
C  KOUNT(3) THE NUMBER OF CALLS AT DTMAX
C  KOUNT(4) THE NUMBER OF CALLS AT DTMIN
C  KOUNT(5) THE NUMBER OF CALLS WITH DTUSED SO SMALL THAT THE ACCURACY
C           OF THE ALGORITHM MAY BE DEGRADED
C
C         IN / BASINT /
C   DTMAX - MAX. STEP SIZE ALLOWED RELATIVE TO ABS(HINTVL).
C             UNINITIALIZED DTMAX IMPLIES DEFAULT VALUE OF 1.0
C   DTMIN - MIN. STEP SIZE ALLOWED RELATIVE TO ABS(HINTVL).
C             UNINITIALIZED DTMIN IMPLIES DEFAULT VALUE OF 1.E-12
C   YCUT  - LOWER BOUND FOR Y IN CALCULATING THE RELATIVE L.E.
C     DYCUT - LOWER BOUND OF SIGNIFICANCE OF DERIVATIVE
C             ERROR EST MUST BE LE EMAX*(AMAX(Y,YCUT,DYCUT-DY))
C        DEFAULTS ARE YCUT=1.OE-14,DYCUT=1.OE-5
C
C
C PROGRAMMED BY -- JAMES LIU           DATE -- JULY, 1977
C
      EXTERNAL RKGET
      REAL Y(N),WS(N,8)
      DIMENSION MES(6)
      LOGICAL LAST,NOTOL,OK
      COMMON/STAT/KOUNT(5)
      COMMON/BASINT/YCUT,DYCUT,DTMAX,DTMIN
      DATA C1,C2,RMAX,RMIN,ANOISE,EMXMIN,U/1.E15,1.E-15,5.,
     ,     0.1,1.E-14,1.E-14,1.E-13/
      DATA MES/2420,2421,2422,2423,2424,2425/
*CALL STATF
C
      IF(INOUT.EQ.0) GO TO 1
      IF(INOUT+3) 170,30,30
C
C       ****************************
C       *** INITIALIZATION ENTRY ***
C       ****************************
C
 1    INOUT=1
C
C        INITIALIZATION OF PARAMETERS
C
      DO 2 I=1,5
 2    KOUNT(I)=0
C
C      THE FOLLOWING VALUES WILL BE DEFINED IN FORSIM
C     BUT NOT NECESSARILY IF CALLED FROM  AELIB LIBRARY WITHOUT FORSIM
C            HENCE CHECK THEM USING LEGVAR
C
      IF (LEGVAR(YCUT).NE.0) YCUT=ANOISE
      IF (LEGVAR(T).NE.0) T=0.
      IF (LEGVAR(EMAX).NE.0) EMAX=1.E-5
      IF (LEGVAR(DTMAX).NE.0) DTMAX=C1
      IF (LEGVAR(DTMIN).NE.0) DTMIN=C2
      IF(LEGVAR(HINTVL).NE.0) HINTVL=0.
      IF (HINTVL.LT.0.) HINTVL=-HINTVL
      IF(LEGVAR(DYCUT).NE.0)DYCUT=1.0E-5
C
C     SET UP THE CORRECT SENCE OF INTEGRATION, FORWARD OR BACKWARD
C
      SIGN=1.
      TEND=T
      DO 5 I=1,N
    5 WS(I,1)=0.0
C
C     CHECK FOR INPUT PARAMETERS
C
      IF((HINTVL.NE.0.).AND.(ABS(HINTVL).LT.DTMIN)) HINTVL=DTMIN
      IF (N.LE.0) GOTO 150
      IF(((EMAX.GT.1.E-2).OR.(EMAX.LT.1.E-10)).AND.(ABS(EMAX).GT.0.)) GO
     , TO 150
      IF((DTMAX.LT.DTMIN).OR.(DTMIN.LE.0.)) GO TO 150
      IF (LEGVAR(DT).EQ.0) GOTO 25
C
C     CALCULATION OF INITIAL STEP SIZE WHEN NOT SPECIFIED BY USER
C
         CALL RKGET(N,T,Y,WS(1,2))
      KOUNT(1)=KOUNT(1)+1
         WS(1,2)=ABS(WS(1,2))
         IF (N.EQ.1) GOTO 15
            DO 10 I=2,N
   10       WS(1,2)=AMAX1(WS(1,2),ABS(WS(I,2)))
   15    IF (WS(1,2).EQ.0.) GOTO 20
            DT=(EMAX / WS(1,2))**0.2
            GOTO 25
   20    DT=0.5*(DTMAX+DTMIN)
   25 DT=ABS(DT)
      GOTO 40
C
C       ********************
C       *** NORMAL ENTRY ***
C       ********************
C
 30   INOUT=1
      IF (HINTVL.EQ.0.) GOTO 40
C
C     RESTORE PREVIOUS VALUES OF T AND Y BEFORE LAST OUTPUT POINT.
C
         T=TSAVE
         DO 35 I=1,N
   35    Y(I)=WS(I,8)
C
C     LAST=.TRUE. INDICATES THAT THIS IS THE LAST STEP IF THE
C     RESULTED ERROR IS ACCEPTABLE.
C     NOTOL=.TRUE. IMPLIES DTMIN IS USED SO THAT NO ERROR CRITERION
C     IS TO BE MET ON THE RESULT
C     OK=.TRUE. INDICATES THAT THE CURRENT STEP TAKEN IS SUCCESSFUL.
C
   40 LAST=.FALSE.
      NOTOL=.FALSE.
      OK=.FALSE.
      TEND=TEND+SIGN*HINTVL
      IF (EMAX.GT.0.) GOTO 135
C
C     DETECTION OF LAST STEP OF INTEGRATION AND SETTING THE LAST
C     STEP SO THAT T WILL FALL EXACTLY ON T+HINTVL.
C
   45 IF (HINTVL.EQ.0.) GOTO 50
         DDT=ABS(TEND-T)
         IF (DDT.GT.DT*(1.+ANOISE)) GOTO 50
            LAST=.TRUE.
            DTLAST=DT
            TSAVE=T
C
C     COMPENSATE FOR ANY LOSS (DUE TO SUBTRACTION AND ADDITION) IN T
C     THAT WILL CAUSE IT TO FALL SHORT OF THE SPECIFIED OUTPUT POINT.
C     THIS SHOULD NOT CAUSE ANY TROUBLE SINCE THE OUTPUT VALUES OF T AND
C     Y ARE NEGLECTED DURING INPUT FOR THE NEXT INTERVAL.
C
            DT=DDT*(1.+ANOISE)
 50   ADT=ABS(DT)
      UDT=U*(ADT+ABS(T))
*IF -DEF,AELIB5
      IF(LAST.OR.ADT.GT.UDT) GO TO 52
*ENDIF
*IF DEF,AELIB5
      IF(LAST.OR.ADT.GT.UDT) GO TO 52
*ENDIF
      KOUNT(5)=KOUNT(5)+1
      DT=UDT*SIGN
      NOTOL=.TRUE.
 52   CONTINUE
C
C ******************************************************
C ******  START OF ONE STEP OF RKF45 INTEGRATION  ******
C ******************************************************
C
C
C     THE FOLLOWING PERFORMS ONE STEP OF THE RUNGE KUTTA FEHLBERG
C     INTEGRATION METHOD
C
C     ON RETURN --
C       ERRMAX CONTAINS THE RELATIVE MAX. L.E. AMONG COMPONENTS OF Y
C       WHILE WS(*,3) CONTAINS THE ABSOLUTE ERROR IN EACH COMPONENT OF
C       Y IN THIS STEP.
C
      SIGNDT=SIGN*DT
      KOUNT(1)=KOUNT(1)+6
      CH=SIGNDT / 180329760.
      SAVET=T
      CALL RKGET(N,T,Y,WS(1,2))
      DO 55 I=1,N
         WS(I,8)=Y(I)
         Y(I)=WS(I,8) + CH * (WS(I,2) * 45082440.)
   55 CONTINUE
      T=SAVET+SIGNDT/4.
      CALL RKGET(N,T,Y,WS(1,3))
      DO 60 I=1,N
         Y(I)=WS(I,8) + CH * (  WS(I,2) * 16905915.
     ,         + WS(I,3) * 50717745.)
   60 CONTINUE
      T=SAVET+SIGNDT*3./8.
      CALL RKGET(N,T,Y,WS(1,4))
      DO 65 I=1,N
         Y(I)=WS(I,8) + CH * (  WS(I,2) * 158578560.
     ,         - WS(I,3) * 590976000.                          + WS(I,4)
     ,    * 598855680.)
   65 CONTINUE
      T=SAVET+SIGNDT*12./13.
      CALL RKGET(N,T,Y,WS(1,5))
      DO 70 I=1,N
         Y(I)=WS(I,8) + CH * (  WS(I,2) * 366503540.
     ,         - WS(I,3) * 1442638080.                         + WS(I,4)
     ,    * 1293593600.                         - WS(I,5) * 37129300.)
   70 CONTINUE
      T=SAVET+SIGNDT
      CALL RKGET(N,T,Y,WS(1,6))
      DO 75 I=1,N
         Y(I)=WS(I,8) + CH * (- WS(I,2) * 53431040.
     ,         + WS(I,3) * 360659520.                          - WS(I,4)
     ,    * 249157376.                          + WS(I,5) * 81684460.
     ,                           - WS(I,6) * 49590684.)
   75 CONTINUE
      T=SAVET+SIGNDT/2.
      CALL RKGET(N,T,Y,WS(1,7))
      DO 80 I=1,N
         Y(I)=WS(I,8) + CH * (  WS(I,2) * 20871500.
     ,         + WS(I,4) * 98988032.                              + WS(I
     ,   ,5) * 96536180.                              - WS(I,6) *
     ,   36065952.)
   80 CONTINUE
C
C     NOW TO CALCULATE THE ERROR ESTIMATE IN THIS STEP.
C
      ERRMAX=0.
      DO 85 I=1,N
         WS(I,3)= - WS(I,2) / 360.
     ,         + WS(I,4) * 128. / 4275.
     ,      + WS(I,5) * 2197. / 75240.
     ,   - WS(I,6) / 50.                                             -
     ,   WS(I,7) * 2. / 55.
         WS(I,3)=ABS(WS(I,3))
         IF (EMAX.EQ.0.) GOTO 85
      YBASE=AMAX1(ABS(Y(I)),YCUT,DYCUT-ABS(WS(I,2)))
            ERRMAX=AMAX1(ERRMAX,WS(I,3)/YBASE)
   85 CONTINUE
      ERRMAX=AMAX1(ERRMAX,EMXMIN)
C
C ****************************************************
C ******  END OF ONE STEP OF RKF45 INTEGRATION  ******
C ****************************************************
C
C
C     EMAX=0 IMPLIES FIXED STEP SIZE. THEREFORE BYPASS THE CALCULATION
C     OF NEXT STEP SIZE, AND NO ERROR CRITERION IS TO BE MET.
C     HOWEVER THE MAX. L.E. WILL STILL BE RECORDED.
C
      IF (EMAX) 100,90,100
   90 DO 95 I=1,N
   95 WS(I,1)=AMAX1(WS(I,1),WS(I,3))
      KOUNT(2)=KOUNT(2)+1
      KFLAG=0
      T=SAVET+SIGNDT
      IF (LAST) GOTO 175
      IF (HINTVL.EQ.0.) GOTO 175
      GOTO 45
C
C **********************************************************************
C     THE FOLLOWING ONLY APPLIES TO VARIABLE STEP SIZE INTEGRATION.
C **********************************************************************
C
C     DECIDE WHETHER TO ACCEPT PREVIOUS STEP OR NOT IN VARIABLE STEP
C     SIZE SETTING. IF STEP SIZE HAS BEEN MIN. STEP SIZE, ACCEPT IT
C     AUTOMATICALLY.
C
  100 IF (.NOT.NOTOL) GOTO 105
         NOTOL=.FALSE.
         GOTO 125
C
C     RESTORE THE PREVIOUS VALUES OF Y (IF STEP IS REJECTED),
C     UPDATE T AND MAX.ERR IN EACH COMPONENT. (IT STEP IS ACCEPTED)
C
  105 IF (ERRMAX.LE.EMAX) GOTO 115
         DO 110 I=1,N
  110    Y(I)=WS(I,8)
         T=SAVET
         LAST=.FALSE.
         OK=.FALSE.
         GOTO 130
  115 DO 120 I=1,N
  120 WS(I,1)=AMAX1(WS(I,1),WS(I,3))
  125 T=SAVET+SIGNDT
      KOUNT(2)=KOUNT(2)+1
      KFLAG=0
      OK=.TRUE.
      IF (LAST) GOTO 175
C
C     CALCULATION AND TESTING ON NEXT STEP SIZE.
C
  130 R=0.9*(EMAX/ERRMAX)**0.2
      R=AMIN1(R,RMAX)
      R=AMAX1(R,RMIN)
      DT=R * DT
  135 IF (DT.GT.DTMIN) GOTO 140
      IF(KFLAG.EQ.-10) GO TO 160
      NOTOL=.TRUE.
      KFLAG=KFLAG-1
      IF(KFLAG.LT.-1) GO TO 140
      DT=DTMIN
      KOUNT(4)=KOUNT(4)+1
  140 IF (DT.LT.DTMAX) GOTO 145
      KOUNT(3)=KOUNT(3)+1
         DT=DTMAX
  145 IF ((HINTVL.EQ.0.).AND.OK) GOTO 175
      GOTO 45
C
C     ERROR PROCESSING
C
 150  INOUT=-5
*IF -DEF,AELIB5
      CALL AELERR(MES(5),2HNF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MES(5),'NF')
*ENDIF
      GO TO 180
 160  INOUT=-4
*IF -DEF,AELIB5
      CALL AELERR(MES(4),2HNF)
*ENDIF
*IF DEF,AELIB5
      CALL AELERR(MES(4),'NF')
*ENDIF
      GO TO 180
C        ENTERED WITH PREVIOUS FATAL ERROR - PROGRAM TERMINATES.
*IF -DEF,AELIB5
 170  CALL AELERR(MES(6),1HF)
*ENDIF
*IF DEF,AELIB5
 170  CALL AELERR(MES(6),'F')
*ENDIF
      GOTO 180
C
C     ***********************
C     ***** NORMAL EXIT *****
C     ***********************
C
 175  IF(KOUNT(3).GT.0) INOUT=-1
      IF(KOUNT(4).GT.0) INOUT=-2
      IF(KOUNT(5).GT.0) INOUT=-3
      IF (HINTVL.NE.0.) DT=DTLAST
      IMES=IABS(INOUT)
*IF -DEF,AELIB5
      IF(INOUT.LT.0.AND.MF.EQ.0) CALL AELERR(MES(IMES),2HNF)
*ENDIF
*IF DEF,AELIB5
      IF(INOUT.LT.0.AND.MF.EQ.0) CALL AELERR(MES(IMES),'NF')
*ENDIF
 180  RETURN
      END
